@@ -8,7 +8,7 @@ from time import perf_counter
 
 from tqdm import tqdm
 
-from covert import passphrase, pubkey, util
+from covert import passphrase, pubkey, tty, util
 from covert.archive import Archive
 from covert.blockstream import decrypt_file, encrypt_file
 
@@ -80,7 +80,9 @@ def run_decryption(infile, args, passwords, identities):
           f = open(name, 'wb')
         elif outdir is None:
           outdir = False
-          progress.write(" ▶️ \x1B[1;34m  The archive contains files. To extract, use \x1B[1;37m-o PATH\x1B[0m", file=stderr)
+          progress.write(
+            " ▶️ \x1B[1;34m  The archive contains files. To extract, use \x1B[1;37m-o PATH\x1B[0m", file=stderr
+          )
 
       # Next file
       if progress:
@@ -179,7 +181,8 @@ def main_enc(args):
       progress.write(f'{s:15,d} 📄 {n:60}' if n else f'{s:15,d} 💬 <message>', file=stderr)
     if not cur:
       a.random_padding(padding)
-      progress.write(f'{a.padding:15,d} ⬛ <padding>', file=stderr)
+      progress.write(f'\x1B[1;30m{a.padding:15,d} ⬛ <padding>\x1B[0m', file=stderr)
+
   a.nextfilecb = nextfile_callback
   # Main processing
   with outf:
@@ -234,28 +237,12 @@ def main_dec(args):
   infile = open(args.files[0], "rb") if args.files else stdin.buffer
   # If ASCII armored or TTY, read all input immediately (assumed to be short enough)
   total_size = os.path.getsize(args.files[0]) if args.files else 0
-  if args.armor or infile.isatty():
-    if infile.isatty():
-      stderr.write("\x1B[?1049h\x1B[1;1H\x1B[1m〰 COVERT 〰\x1B[0m   (paste and press enter)\x1B[1;30m\n")
-      stderr.flush()
-    try:
-      data = b""
-      while True:
-        line = input().strip().encode()
-        if not line: break
-        data += line
-      data = util.armor_decode(data)
-      total_size = len(data)
-    except Exception as e:
-      raise ValueError(f"Incomplete or malformed data. {e} {len(data)}\n{line}")
-    finally:
-      with suppress(Exception):
-        if infile.isatty():
-          stderr.write("\x1B[0m\x1B[2J\x1B[?1049l")
-          stderr.flush()
+  if infile.isatty():
+    data = util.armor_decode(tty.read_hidden("Encrypted message").encode())
     if not data:
-      raise ValueError("No input, aborted.")
+      raise KeyboardInterrupt
     infile = BytesIO(data)
+    total_size = len(data)
     del data
   elif 40 <= total_size <= ARMOR_MAX_SIZE:
     # Try reading the file as armored text rather than binary
